// Channels
import { TCP } from "../channels/TCP.js"
import { HTTP } from "../channels/HTTP.js"
import { File } from "../channels/File.js"
import { MQTT } from "../channels/MQTT.js"
import { Satcom } from "../channels/Satcom.js"
import { Bluetooth } from "../channels/Bluetooth.js"

// Protocols
import { TAIP } from "../protocols/TAIP.js"
import { Json } from "../protocols/Json.js"
import { CSV } from "../protocols/CSV.js"


/**
 * Creates a destination (which is basically a channel-protocol pair) with all the config passed in the syruslang
 *
 * @param {*} config all the parameters from syruslang to create the destination like protocol, port, host, name, etc
 */
export function createDestination(config, callback) {
  let { name, protocol, channel, host, port, user, password } = config
  // let { headers, params, url } = config
  let { ack, disabled } = config
  let client, parser
  // set default ack
  if (!ack) {
    ack = "seq"
    if (!"tcp,mqtt".includes(channel)) ack = "disabled"
  }

  // function to procces syruslang commands from every destination (as TAIP receives STX)
  // this is passed to every destination

  /** Channels Creation */
  if (channel == "tcp") {
    client = new TCP(config);
  } else if (channel == "file") {
    client = new File(config); //{ host }
  } else if (channel.startsWith('http')) {
    client = new HTTP(config);
  } else if (channel == "mqtt") {
    config.onCommand = callback;
    client = new MQTT(config);
  } else if (channel == "satcom"){
    client = new Satcom(config)
  } else if (channel == "bluetooth"){
    client = new Bluetooth(config)
  }

  if (disabled) client.disable()

  /** Protocols Creation */
  if (protocol == "taip") {
    parser = new TAIP({
      name: name,
      client: client,
      ack: ack,
      onCommand: callback,
    });
  } else if (protocol == "json") {
    parser = new Json({
      name: name,
      client: client,
      onCommand: callback,
    });
  } else if (protocol == "csv") {
    parser = new CSV({
      name: name,
      client: client, 
      onCommand: callback
    });
  }

  // The destination object allow to easily verify if the destination exists and what events belong to it
  return {
    client: client,
    parser: parser,
    protocol: protocol,
    channel: channel,
    host: host,
    port: port,
    user: user,
    password: password,
    // sendEvent is called for the main module everytime signals are up and the event must be fired
    sendEvent(data, event, fields, label, code) {
      this.parser.send(data, event, fields, label, code);
    },
    // callback called when the app is about to die, to make clean disconnection (as mqtt for example send the last will)
    disconnect() { client.disconnect() }
  };
}
